# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright 2021 InferStat Ltd
# Created by: Joshua Mason
# Created date: 11/03/2021

"""
Functionality to adapt external libraries for usage with InferTrade.
"""
from infertrade.algos.external.ta_regressions import ta_adaptor, ta_export_regression_allocations, ta_export_signals
from infertrade.PandasEnum import PandasEnum

ta_export = {PandasEnum.SIGNAL.value: ta_export_signals, PandasEnum.ALLOCATION.value: ta_export_regression_allocations}



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Author: Thomas Oliver
# Creation date: 11th March 2021
# Copyright 2021 InferStat Ltd


"""
Example calculations of returns using InferTrade functions.
"""

from pathlib import Path
import pandas as pd
import matplotlib.pyplot as plt
from infertrade.utilities.performance import calculate_portfolio_performance_python
import matplotlib


def buy_on_small_rises(df: pd.DataFrame) -> pd.DataFrame:
    """A rules that buys when the market rose between 2% and 10% from previous close."""
    df["allocation"] = 0.0
    df.loc[df["price"].pct_change(50) >= 0.02, "allocation"] = 0.25
    df.loc[df["price"].pct_change(50) >= 0.05, "allocation"] = 0.5
    df.loc[df["price"].pct_change(50) >= 0.10, "allocation"] = 0.0
    return df


if __name__ == "__main__":
    # Example script - import Gold prices and apply the buy_on_small_rises algorithm and plot.
    lbma_gold_location = Path(Path(__file__).absolute().parent, "LBMA_Gold.csv")
    my_dataframe = pd.read_csv(lbma_gold_location)
    my_dataframe_without_allocations = my_dataframe.rename(columns={"LBMA/GOLD usd (pm)": "price", "Date": "date"})
    my_dataframe_with_allocations = buy_on_small_rises(my_dataframe_without_allocations)
    my_dataframe_with_returns = calculate_portfolio_performance_python(my_dataframe_with_allocations)
    my_dataframe_with_returns.plot(x="date", y=["allocation", "portfolio_return"])
    plt.show()



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#     http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Created by: Thomas Oliver
# Created date: 18th March 2021
# Copyright 2021 InferStat Ltd

"""
API facade that allows interaction with the library with strings and vanilla Python objects.
"""

# Python standard library
from typing import List, Union
from copy import deepcopy
import pandas as pd

# InferTrade packages
from infertrade.algos import algorithm_functions, ta_adaptor
from infertrade.utilities.operations import ReturnsFromPositions
from infertrade.PandasEnum import PandasEnum


class Api:
    """All public methods should input/output json-serialisable dictionaries."""

    @staticmethod
    def get_allocation_information() -> dict:
        """Provides information on algorithms that calculate positions."""
        combined_data = {}
        for ii_package in algorithm_functions:
            combined_data.update(algorithm_functions[ii_package][PandasEnum.ALLOCATION.value])
        return combined_data

    @staticmethod
    def get_signal_information() -> dict:
        """Provides information on algorithms that calculate signals."""
        combined_data = {}
        for ii_package in algorithm_functions:
            combined_data.update(algorithm_functions[ii_package][PandasEnum.SIGNAL.value])
        return combined_data

    @staticmethod
    def get_algorithm_information() -> dict:
        """Provides information on algorithms (signals and positions) as flat list (not nested by category)."""
        combined_allocation_data = Api.get_allocation_information()
        combined_signal_data = Api.get_signal_information()
        combined_data = {}
        combined_data.update(combined_allocation_data)
        combined_data.update(combined_signal_data)
        return combined_data

    @staticmethod
    def available_packages() -> List[str]:
        """Returns the list of supported packages."""
        return list(algorithm_functions.keys())

    @staticmethod
    def return_algorithm_category(algorithm_name: str) -> str:
        """Returns the category of algorithm as a string."""
        if algorithm_name in Api.get_signal_information():
            algo_type = PandasEnum.SIGNAL.value
        elif algorithm_name in Api.get_allocation_information():
            algo_type = PandasEnum.ALLOCATION.value
        else:
            raise NameError("Algorithm is not supported: ", algorithm_name)
        return algo_type

    @staticmethod
    def algorithm_categories() -> List[str]:
        """Returns the list of algorithm types."""
        return [PandasEnum.ALLOCATION.value, PandasEnum.SIGNAL.value]

    @staticmethod
    def available_algorithms(
        filter_by_package: Union[str, List[str]] = None, filter_by_category: Union[str, List[str]] = None
    ) -> List[str]:
        """Returns a list of strings that are available strategies."""
        if not filter_by_package:
            filter_by_package = Api.available_packages()
        elif isinstance(filter_by_package, str):
            filter_by_package = [filter_by_package]

        if not filter_by_category:
            filter_by_category = Api.algorithm_categories()
        elif isinstance(filter_by_category, str):
            filter_by_category = [filter_by_category]

        names = []
        for ii_package in filter_by_package:
            for jj_type in filter_by_category:
                algorithms = list(algorithm_functions[ii_package][jj_type].keys())
                names += algorithms
        return names

    @staticmethod
    def determine_package_of_algorithm(name_of_algorithm: str) -> str:
        """Determines the original package of a strategy."""
        category = Api.return_algorithm_category(name_of_algorithm)
        package_name = "Unknown"
        for ii_package in Api.available_packages():
            algo_list = Api.available_algorithms(filter_by_package=ii_package, filter_by_category=category)
            if name_of_algorithm in algo_list:
                package_name = ii_package
        return package_name

    @staticmethod
    def required_inputs_for_algorithm(name_of_strategy: str) -> List[str]:
        """Describes the input columns needed for the strategy."""
        full_info = Api.get_algorithm_information()
        required_inputs = full_info[name_of_strategy]["series"]
        return required_inputs

    @staticmethod
    def required_parameters_for_algorithm(name_of_strategy: str) -> List[str]:
        """Describes the input columns needed for the strategy."""
        full_info = Api.get_algorithm_information()
        required_inputs = full_info[name_of_strategy]["parameters"]
        return required_inputs

    @staticmethod
    def _get_raw_callable(name_of_strategy_or_signal: str) -> callable:
        """Private method to return the raw function - should not be used externally."""
        info = Api.get_algorithm_information()
        callable_fields = ["function", "class"]

        try:
            callable_key = next(key for key in callable_fields if key in info[name_of_strategy_or_signal])
            raw_callable = info[name_of_strategy_or_signal][callable_key]

        except StopIteration:
            raise KeyError("The dictionary has no recognised callable (" + ",".join(callable_fields) + ") fields.")

        except KeyError:
            raise KeyError("A strategy or signal was requested that could not be found: ", name_of_strategy_or_signal)

        if callable_key is "function":
            # We will not amend.
            callable_func = raw_callable

        else:
            # Is a class, so we need to adapt, so we call the adaptor.
            package_of_algo = Api.determine_package_of_algorithm(name_of_strategy_or_signal)
            function_name = info[name_of_strategy_or_signal]["function_names"]
            if package_of_algo is "ta":
                callable_func = ta_adaptor(raw_callable, function_name)
            else:
                raise NotImplementedError("An adapter for this class type has not yet been created.")

        return callable_func

    @staticmethod
    def calculate_allocations(
        df: pd.DataFrame, name_of_strategy: str, name_of_price_series: str = PandasEnum.MID.value
    ) -> pd.DataFrame:
        """Calculates the allocations using the supplied strategy."""
        if name_of_price_series is not PandasEnum.MID.value:
            df[PandasEnum.MID.value] = df[name_of_price_series]
        rule_function = Api._get_raw_callable(name_of_strategy)
        df_with_positions = rule_function(df)
        return df_with_positions

    @staticmethod
    def calculate_returns(df: pd.DataFrame) -> pd.DataFrame:
        """Calculates the returns from supplied positions."""
        df_with_returns = ReturnsFromPositions().transform(df)
        return df_with_returns

    @staticmethod
    def calculate_allocations_and_returns(
        df: pd.DataFrame, name_of_strategy: str, name_of_price_series: str = PandasEnum.MID.value
    ) -> pd.DataFrame:
        """Calculates the returns using the supplied strategy."""
        df_with_positions = Api.calculate_allocations(df, name_of_strategy, name_of_price_series)
        df_with_returns = ReturnsFromPositions().transform(df_with_positions)
        return df_with_returns

    @staticmethod
    def calculate_signal(df: pd.DataFrame, name_of_signal: str) -> pd.DataFrame:
        """Calculates the allocations using the supplied strategy."""
        class_of_signal_generator = Api()._get_raw_callable(name_of_signal)
        original_df = deepcopy(df)
        df_with_signal = class_of_signal_generator(original_df)
        return df_with_signal

    @staticmethod
    def get_available_representations(name_of_algorithm: str) -> List[str]:
        """Describes which representations exist for the algorithm."""
        dict_of_properties = Api.get_algorithm_information()
        try:
            available_representations = list(
                dict_of_properties[name_of_algorithm]["available_representation_types"].keys()
            )
        except KeyError:
            raise NotImplementedError("The requested algorithm does not have any representations: ", name_of_algorithm)
        return available_representations

    @staticmethod
    def return_representations(
        name_of_algorithm: str, representation_or_list_of_representations: Union[str, List[str]] = None
    ) -> dict:
        """Returns the representations (e.g. URLs of relevant documentation)."""

        dict_of_properties = Api.get_algorithm_information()
        try:
            representations = dict_of_properties[name_of_algorithm]["available_representation_types"]
        except KeyError:
            raise NotImplementedError("The requested algorithm does not have any representations: ", name_of_algorithm)

        if not representation_or_list_of_representations:
            representation_or_list_of_representations = Api.get_available_representations(name_of_algorithm)

        if isinstance(representation_or_list_of_representations, str):
            representation_or_list_of_representations = [representation_or_list_of_representations]

        if isinstance(representation_or_list_of_representations, list):
            representation_dict = {}
            for ii_entry in representation_or_list_of_representations:
                if ii_entry in representations:
                    representation_dict.update({ii_entry: representations[ii_entry]})
                else:
                    raise NameError("Could not find this representation: ", ii_entry)
        else:
            raise TypeError("Input type not supported: ", representation_or_list_of_representations)

        return representation_dict

    @staticmethod
    def export_to_csv(
        dataframe: pd.DataFrame,
        rule_name: str,
        string_return: bool = False,
        relationship: str = None,
        second_df: pd.DataFrame = None,
    ):
        """
        Exports csv file of calculated portfolio performance based on data gained from calculating trading rules and
         relationships.
        """
        from infertrade.utilities.export import export_performance_df

        # imported inside to prevent circular dependency
        df = export_performance_df(
            dataframe=dataframe, second_df=second_df, rule_name=rule_name, relationship=relationship
        )
        if string_return is True:
            return df.to_csv()
        else:
            if relationship is not None:
                return df.to_csv(str(rule_name) + "_performance_" + str(relationship) + ".csv")
            else:
                return df.to_csv(str(rule_name) + ".csv")



## Copyright 2021 InferStat Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Data for LBMA Gold price (USD PM fixing) obtained via Quandl.com from the London Bullion Market Association
# https://www.quandl.com/data/LBMA-London-Bullion-Market-Association
#
# Author: Thomas Oliver
# Creation date: 11th March 2021

"""
Directory for example scripts showing usages of InferTrade.
"""



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright 2021 InferStat Ltd
# Created by: Thomas Oliver
# Created date: 23/08/2021

# External packages
from collections import Callable

from infertrade.PandasEnum import PandasEnum
import pandas as pd
from sklearn.preprocessing import FunctionTransformer
from sklearn.pipeline import make_pipeline


# InferStat imports
from infertrade.algos.external.ta import ta_export_signals
from infertrade.utilities.operations import (
    PositionsFromPricePrediction,
    PricePredictionFromSignalRegression,
)
from infertrade.algos.external.ta import ta_adaptor


def create_allocation_function(ta_signal_name: str, ta_raw_signal_func: Callable) -> Callable:
    """Creates an allocation function for a ta library signal by regression against the time step price changes."""

    def allocation_function(time_series_df: pd.DataFrame, *args, **kwargs) -> pd.DataFrame:
        """Generic regression treatment of ta technical signals."""

        adapted_allocation_rule_using_regression = ta_adaptor(ta_raw_signal_func, ta_signal_name, *args, **kwargs)

        pipeline = make_pipeline(
            FunctionTransformer(adapted_allocation_rule_using_regression),
            PricePredictionFromSignalRegression(),
            PositionsFromPricePrediction(),
        )

        time_series_df = pipeline.fit_transform(time_series_df)
        return time_series_df

    return allocation_function


github_permalink = "https://github.com/ta-oliver/infertrade/blob/e45fe2b76f9a16c07c48a6739e3173737cb5355c/infertrade/algos/community/ta_regressions.py"


def ta_rules_with_regression() -> dict:
    """Creates an equivalent dictionary of allocation rules by regressing signals against future price changes."""

    allocation_dictionary = {}

    for ii_ta_signal in ta_export_signals:
        ta_rule_name = ta_export_signals[ii_ta_signal]["function_names"]

        ta_regression_name = ta_rule_name + "_regression"

        ta_raw_signal_func = ta_export_signals[ii_ta_signal]["class"]

        dictionary_addition = {
            ta_regression_name: {
                "function": create_allocation_function(ta_rule_name, ta_raw_signal_func),
                "parameters": ta_export_signals[ii_ta_signal]["parameters"],
                "series": ta_export_signals[ii_ta_signal]["series"],
                "available_representation_types": {
                    "github_permalink": github_permalink
                    + "#L"
                    + str(create_allocation_function.__code__.co_firstlineno)
                },
            }
        }

        allocation_dictionary.update(dictionary_addition)

    return allocation_dictionary


# Export the regression format rules.
ta_export_regression_allocations = ta_rules_with_regression()



data_dictionary = {
    "fifty_fifty": {
        "function": "<function fifty_fifty at 0x102bb07b8>",
        "parameters": {},
        "series": [],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L38"
        },
    },
    "buy_and_hold": {
        "function": "<function buy_and_hold at 0x11944aa60>",
        "parameters": {},
        "series": [],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L44"
        },
    },
    "chande_kroll_crossover_strategy": {
        "function": "<function chande_kroll_crossover_strategy at 0x11944aae8>",
        "parameters": {},
        "series": ["high", "low", "price"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L50"
        },
    },
    "change_relationship": {
        "function": "<function change_relationship at 0x11944ac80>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L82"
        },
    },
    "change_relationship_oos": {
        "function": "change_relationship_OOS at 0x7f0a4fb86200>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/414caf29aed7b8812fd0a71e0ea12d9fdd1c2951/infertrade"
                "/algos/community/allocations.py#L103"
        },
    },
    "combination_relationship": {
        "function": "<function combination_relationship at 0x11951ca60>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L131"
        },
    },
    "combination_relationship_oos": {
        "function": "<function combination_relationship_OOS at 0x7f0a4fd6dcb0>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/414caf29aed7b8812fd0a71e0ea12d9fdd1c2951/infertrade"
                "/algos/community/allocations.py#L181"
        },
    },
    "difference_relationship": {
        "function": "<function difference_relationship at 0x11951cbf8>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L203"
        },
    },
    "difference_relationship_oos": {
        "function": "<function difference_relationship_OOS at 0x7f0a4fd6def0>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/414caf29aed7b8812fd0a71e0ea12d9fdd1c2951/infertrade"
                "/algos/community/allocations.py#L281"
        },
    },
    "level_relationship": {
        "function": "<function level_relationship at 0x11951cd90>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L269"
        },
    },
    "level_relationship_oos": {
        "function": "<function level_relationship_OOS at 0x7f0a4fd7f170>",
        "parameters": {},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/414caf29aed7b8812fd0a71e0ea12d9fdd1c2951/infertrade"
                "/algos/community/allocations.py#L372"
        },
    },
    "constant_allocation_size": {
        "function": "<function constant_allocation_size at 0x11951cb70>",
        "parameters": {"fixed_allocation_size": 1.0},
        "series": [],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L192"
        },
    },
    "high_low_difference": {
        "function": "<function high_low_difference at 0x11951cd08>",
        "parameters": {"scale": 1.0, "constant": 0.0},
        "series": ["high", "low"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L256"
        },
    },
    "sma_crossover_strategy": {
        "function": "<function sma_crossover_strategy at 0x11951cea0>",
        "parameters": {"fast": 0, "slow": 0},
        "series": ["price"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L317"
        },
    },
    "weighted_moving_averages": {
        "function": "<function weighted_moving_averages at 0x11951cf28>",
        "parameters": {
            "avg_price_coeff": 1.0,
            "avg_research_coeff": 1.0,
            "avg_price_length": 2,
            "avg_research_length": 2,
        },
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L337"
        },
    },
    "change_regression": {
        "function": "<function change_regression at 0x119520048>",
        "parameters": {"change_coefficient": 0.1, "change_constant": 0.1},
        "series": ["research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L380"
        },
    },
    "difference_regression": {
        "function": "<function difference_regression at 0x1195200d0>",
        "parameters": {"difference_coefficient": 0.1, "difference_constant": 0.1},
        "series": ["price", "research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L396"
        },
    },
    "level_regression": {
        "function": "<function level_regression at 0x119520158>",
        "parameters": {"level_coefficient": 0.1, "level_constant": 0.1},
        "series": ["research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L414"
        },
    },
    "level_and_change_regression": {
        "function": "<function level_and_change_regression at 0x1195201e0>",
        "parameters": {"level_coefficient": 0.1, "change_coefficient": 0.1, "level_and_change_constant": 0.1},
        "series": ["research"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L431"
        },
    },
    "buy_golden_cross_sell_death_cross": {
        "function": "<function buy_golden_cross_sell_death_cross at 0x119520268>",
        "parameters": {
            "allocation_size": 0.5,
            "deallocation_size": 0.5,
            "short_term_moving_avg_length": 50,
            "long_term_moving_avg_length": 200,
        },
        "series": ["price"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L457"
        },
    },
    "SMA_strategy": {
        "function": "<function SMA_strategy at 0x1195202f0>",
        "parameters": {"window": 1, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L490"
        },
    },
    "WMA_strategy": {
        "function": "<function WMA_strategy at 0x119520378>",
        "parameters": {"window": 1, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L504"
        },
    },
    "MACD_strategy": {
        "function": "<function MACD_strategy at 0x119520400>",
        "parameters": {"window_slow": 26, "window_fast": 12, "window_signal": 9, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L519"
        },
    },
    "RSI_strategy": {
        "function": "<function RSI_strategy at 0x119520488>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L536"
        },
    },
    "stochastic_RSI_strategy": {
        "function": "<function stochastic_RSI_strategy at 0x119520510>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L553"
        },
    },
    "EMA_strategy": {
        "function": "<function EMA_strategy at 0x119520598>",
        "parameters": {"window": 50, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L572"
        },
    },
    "bollinger_band_strategy": {
        "function": "<function bollinger_band_strategy at 0x119520620>",
        "parameters": {"window": 20, "window_dev": 2, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L586"
        },
    },
    "PPO_strategy": {
        "function": "<function PPO_strategy at 0x119520730>",
        "parameters": {"window_slow": 26, "window_fast": 12, "window_signal": 9, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L646"
        },
    },
    "PVO_strategy": {
        "function": "<function PVO_strategy at 0x1195207b8>",
        "parameters": {"window_slow": 26, "window_fast": 12, "window_signal": 9, "max_investment": 0.1},
        "series": ["volume"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L662"
        },
    },
    "TRIX_strategy": {
        "function": "<function TRIX_strategy at 0x119520840>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L678"
        },
    },
    "TSI_strategy": {
        "function": "<function TSI_strategy at 0x1195208c8>",
        "parameters": {"window_slow": 25, "window_fast": 13, "window_signal": 13, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L692"
        },
    },
    "STC_strategy": {
        "function": "<function STC_strategy at 0x119520950>",
        "parameters": {
            "window_slow": 50,
            "window_fast": 23,
            "cycle": 10,
            "smooth1": 3,
            "smooth2": 3,
            "max_investment": 0.1,
        },
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L709"
        },
    },
    "KAMA_strategy": {
        "function": "<function KAMA_strategy at 0x1195209d8>",
        "parameters": {"window": 10, "pow1": 2, "pow2": 30, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L736"
        },
    },
    "aroon_strategy": {
        "function": "<function aroon_strategy at 0x119520a60>",
        "parameters": {"window": 25, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L755"
        },
    },
    "ROC_strategy": {
        "function": "<function ROC_strategy at 0x119520ae8>",
        "parameters": {"window": 12, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L776"
        },
    },
    "ADX_strategy": {
        "function": "<function ADX_strategy at 0x119520b70>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close", "high", "low"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L791"
        },
    },
    "vortex_strategy": {
        "function": "<function vortex_strategy at 0x119520bf8>",
        "parameters": {"window": 14, "max_investment": 0.1},
        "series": ["close", "high", "low"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L823"
        },
    },
    "DPO_strategy": {
        "function": "<function DPO_strategy at 0x1195206a8>",
        "parameters": {"window": 20, "max_investment": 0.1},
        "series": ["close"],
        "available_representation_types": {
            "github_permalink":
                "https://github.com/ta-oliver/infertrade/blob/7b8b24bafd1b0a5ba46ba5481432501ea4d83234/infertrade"
                "/algos/community/allocations.py#L632"
        },
    },
}



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# Copyright 2021 InferStat Ltd
# Created by: Joshua Mason
# Created date: 11/03/2021

"""
External algorithms directory.
"""

from infertrade.algos.community import infertrade_export
from infertrade.algos.external import ta_export, ta_adaptor

# A dictionary providing the list of available rules from InferTrade's community rules and external packages.
algorithm_functions = {"infertrade": infertrade_export, "ta": ta_export}
